#define MYSQL_CASE      1
#define EOF_CASE        2

//#define TEST_CASE       MYSQL_CASE

//#define DICT_CACHE      0

#include <Foundation/Foundation.h>
#include <EOAccess/EOAccess.h>
#include <EOControl/EOControl.h>
#if TEST_CASE == MYSQL_CASE
#include <mysql/mysql.h>
#include <mysql/errmsg.h>
#endif

#if TEST_CASE == EOF_CASE
static EOEditingContext *ec = nil;
#if DICT_CACHE
static NSMutableDictionary *dictionary = nil;
#endif
#endif

#if TEST_CASE == MYSQL_CASE
MYSQL db;
#endif

#if TEST_CASE == MYSQL_CASE
static int
DoDBOpen (void)
{
    mysql_init(&db);
    if (!mysql_real_connect(&db,
                            NULL,
                            "root",
                            "",
                            "UAS",
                            0,
                            "/var/run/mysqld/mysqld.sock",
                            0)) {
        return -1;
    }

    return 0;
}

static int
DoDBWork (void)
{
    MYSQL_RES *res;
    MYSQL_ROW row;
    int i;
    char rchBuf[512];
    
    sprintf(rchBuf, "SELECT * FROM ACT_TBL WHERE NAME_COL='chunsj'");

    if ((i = mysql_ping(&db))) {
        switch (i) {
        case CR_COMMANDS_OUT_OF_SYNC:
            printf("[INFO] out of sync\n");
            break;
        case CR_SERVER_GONE_ERROR:
            printf("[INFO] gone\n");
            break;
        case CR_UNKNOWN_ERROR:
            printf("[INFO] unknown error\n");
            break;
        default:
            printf("[ERROR] OOPS!!!\n");
        }
        return -1;
    }

    if (mysql_query(&db, rchBuf)) {
        return -2;
    }

    if (!(res = mysql_store_result(&db))) {
        return -3;
    }

    if (!(row = mysql_fetch_row(res))) {
        mysql_free_result(res);
        return -4;
    }

    printf("[INFO] %s\n", row[3]);

    mysql_free_result(res);

    return 0;
}
#endif

#if TEST_CASE == EOF_CASE
static void
DoDBWork (void)
{
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
    EOGenericRecord *rec;
    id name;

#if DICT_CACHE
    if ((name = [dictionary objectForKey:@"CachedData"])) {
        printf("[INFO] %s\n", [name cString]);
        [pool release];
        return;
    }
#endif

    rec = [ec objectMatchingValue:@"chunsj"
        forKey:@"userName"
        entityNamed:@"Account"];

    if (!ec) {
        printf("[WARN] cannot find record\n");
        return;
    }

#if 1
    printf("[INFO] WinName: %s\n",
        [[rec valueForKey:@"windowsUserName"] cString]);
#if DICT_CACHE
    [dictionary setObject:[rec valueForKey:@"windowsUserName"] 
        forKey:@"CachedData"];
#endif
#endif

    [pool release];
}

static void
DoDBOpen (void)
{
    EODatabase *database;
    EODatabaseContext *context;
    EOModelGroup *group = [[EOModelGroup alloc] init];
    EOModel *model = [group
        addModelWithFile:@"./UAS.eomodeld"];
    if (!model) {
        printf("[WARN] cann open model\n");
        return;
    }

    [EOModelGroup setDefaultGroup:group];
    [group release];

    database = [[[EODatabase alloc] initWithModel:model] autorelease];
    context = [[[EODatabaseContext alloc] initWithDatabase:database]
        autorelease];

    [[EOObjectStoreCoordinator defaultCoordinator]
        addCooperatingObjectStore:context];

    ec = [[EOEditingContext alloc] init];

#if DICT_CACHE
    dictionary = [[NSMutableDictionary alloc] init];
#endif

    printf("[INFO] db setup ok\n");
    return;
}
#endif

int
main (int argc, const char *argv[])
{
    int i;
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];

    DoDBOpen();
    
    for (i = 0; i < 1000; i++) {
        DoDBWork();
    }

    [pool release];
    return 0;
}
